import calendarStyles from "react-big-calendar/lib/css/react-big-calendar.css";
import eventStyles from "../styles/events-team.css";
import { Calendar, dayjsLocalizer } from "react-big-calendar";
import dayjs from "dayjs";
import { type LinksFunction } from "@remix-run/cloudflare";
import {
  Accordion,
  AccordionButton,
  AccordionIcon,
  AccordionItem,
  AccordionPanel,
  Box,
  Container,
} from "@chakra-ui/react";
import { useLoaderData } from "@remix-run/react";
import { useState } from "react";
import utc from "dayjs/plugin/utc.js";

export const links: LinksFunction = () => {
  return [
    { href: calendarStyles, rel: "stylesheet" },
    { href: eventStyles, rel: "stylesheet" },
  ];
};

export async function loader({ context }: { context: RequestContext }) {
  if (!context.data.current_user)
    throw new Response(null, {
      status: 401,
    });

  if (
    ![1 << 3, 1 << 4, 1 << 12].find(
      (p) => context.data.current_user.permissions & p,
    )
  )
    throw new Response(null, {
      status: 403,
    });

  const now = new Date();
  const eventsData = await context.env.D1.prepare(
    "SELECT answer, approved, created_by, day, details, id, month, pending, performed_at, reached_minimum_player_count, type, year FROM events WHERE month = ? AND year = ? ORDER BY day;",
  )
    .bind(now.getUTCMonth() + 1, now.getUTCFullYear())
    .all();

  if (eventsData.error)
    throw new Response(null, {
      status: 500,
    });

  const calendarData = eventsData.results.map((e) => {
    return {
      id: e.id,
      title: (e.type as string).toUpperCase(),
      allDay: true,
      // A Date object will not survive being passed to the client
      start: `${e.year}-${(e.month as number).toString().padStart(2, "0")}-${(e.day as number).toString().padStart(2, "0")}T00:00:00.000Z`,
      end: `${e.year}-${(e.month as number).toString().padStart(2, "0")}-${(e.day as number).toString().padStart(2, "0")}T00:00:00.000Z`,
    };
  });

  return {
    calendarData,
    eventList: eventsData.results,
  };
}

export default function () {
  const data = useLoaderData<typeof loader>();
  const [selectedDate, setDate] = useState(new Date());

  const eventsOfDay = () => {
    return data.eventList.filter(
      (e) =>
        e.day === selectedDate.getUTCDate() &&
        e.month === selectedDate.getUTCMonth() + 1 &&
        e.year === selectedDate.getUTCFullYear(),
    ) as { [k: string]: any }[];
  };

  dayjs.extend(utc);

  return (
    <Container maxW="container.lg" h="600px">
      <Calendar
        endAccessor={(event) => new Date(event.end)}
        events={data.calendarData}
        localizer={dayjsLocalizer(dayjs)}
        onSelectSlot={(s) => {
          setDate(s.slots.at(0) as Date);
        }}
        popup
        startAccessor={(event) => {
          const date = new Date(event.start);
          date.setUTCDate(date.getUTCDate() + 1);

          return date;
        }}
        style={{ height: 500 }}
        views={{
          month: true,
          week: false,
          work_week: false,
          day: false,
          agenda: false,
        }}
      />
      <Accordion id="events-accordion" mt="16px">
        <AccordionItem>
          <h2>
            <AccordionButton>
              <Box as="span" flex="1" textAlign="left">
                Fact of the Day
              </Box>
              <AccordionIcon />
            </AccordionButton>
          </h2>
          <AccordionPanel pb={4}>
            {eventsOfDay().find((e) => e.type === "fotd")?.details || "None"}
            <br />
          </AccordionPanel>
        </AccordionItem>
        <AccordionItem>
          <h2>
            <AccordionButton>
              <Box as="span" flex="1" textAlign="left">
                Gamenight
              </Box>
              <AccordionIcon />
            </AccordionButton>
          </h2>
          <AccordionPanel pb={4}>
            {eventsOfDay().find((e) => e.type === "gamenight")?.details ||
              "None"}
          </AccordionPanel>
        </AccordionItem>
        <AccordionItem>
          <h2>
            <AccordionButton>
              <Box as="span" flex="1" textAlign="left">
                Riddle of the Week
              </Box>
            </AccordionButton>
          </h2>
          <AccordionPanel pb={4}>
            {eventsOfDay().find((e) => e.type === "rotw")?.details || "None"}
          </AccordionPanel>
        </AccordionItem>
        <AccordionItem>
          <h2>
            <AccordionButton>
              <Box as="span" flex="1" textAlign="left">
                Question of the Day
              </Box>
            </AccordionButton>
          </h2>
          <AccordionPanel pb={4}>
            {eventsOfDay().find((e) => e.type === "qotd")?.details || "None"}
          </AccordionPanel>
        </AccordionItem>
      </Accordion>
    </Container>
  );
}
